--------------------------------
-- @module init

--[[--

yy framework 初始化

载入 yy framework 预定义的常量、模块。

### 常量

在初始化框架之前，可以定义以下常量：

-   DEBUG: 设置框架的调试输出级别

    ```lua
    DEBUG = 0           -- 不输出任何调试信息（默认值）
    DEBUG = 1           -- 输出基本的调试信息
    DEBUG = 2           -- 输出详细的调试信息
    ```

-   DEBUG_FPS: 设置是否在画面中显示渲染帧率等信息

    ```lua
    DEBUG_FPS = false   -- 不显示（默认值）
    DEBUG_FPS = true    -- 显示
    ```

-   DEBUG_MEM: 设置是否输出内存占用信息

    ```lua
    DEBUG_MEM = false   -- 不输出（默认值）
    DEBUG_MEM = true    -- 每 10 秒输出一次
    ```
<br />

### 自动载入的模块

框架初始化时，会自动载入以下基本模块：

-   debug: 调试接口
-   functions: 提供一组常用的函数，以及对 Lua 标准库的扩展
-   cocos2dx: 对 cocos2d-x C++ 接口的封装和扩展
-   device: 针对设备接口的扩展
-   display: 创建场景、图像、动画的接口
-   audio: 音乐和音效的接口
-   network: 网络相关的接口
-   crypto: 加密相关的接口
-   json: JSON 的编码和解码接口
-   cc: quick 框架扩展的基础类和组件

]]

print("===========================================================")
print("              LOAD YY FRAMEWORK")
print("===========================================================")

if type(DEBUG) ~= "number" then DEBUG = 0 end
if type(DEBUG_FPS) ~= "boolean" then DEBUG_FPS = false end
if type(DEBUG_MEM) ~= "boolean" then DEBUG_MEM = false end

----
local CURRENT_MODULE_NAME = ...

cc = cc or {}
cc.PACKAGE_NAME = string.sub(CURRENT_MODULE_NAME, 1, -6)

require("cocosext.msg.init")
require("cocosext.language.init")
require("cocosext.common.init")
require("cocosext.net.init")

printInfo("")
printInfo("# DEBUG = "..DEBUG)
printInfo("#")

local sharedTextureCache = cc.Director:getInstance():getTextureCache()
local sharedDirector = cc.Director:getInstance()

-- FPS
if DEBUG_FPS then
    sharedDirector:setDisplayStats(true)
else
    sharedDirector:setDisplayStats(false)
end

-- 内存信息情况
if DEBUG_MEM then
    local sharedTextureCache = cc.Director:getInstance():getTextureCache()
    --[[--
    @ignore
    ]]
    local function showMemoryUsage()
        printInfo(string.format("LUA VM MEMORY USED: %0.2f KB", collectgarbage("count")))
        printInfo(sharedTextureCache:getCachedTextureInfo())
        printInfo("---------------------------------------------------")
    end
    sharedDirector:getScheduler():scheduleScriptFunc(showMemoryUsage, DEBUG_MEM_INTERVAL or 10.0, false)
end
